/*
 * GlushkovTraversal.h
 *
 *  Created on: 13. 3. 2014
 *      Author: tomas
 */

#ifndef GLUSHKOVTRAVERSAL_H_
#define GLUSHKOVTRAVERSAL_H_

#include <set>

#include <regexp/RegExp.h>
#include <regexp/RegExpElement.h>
#include <regexp/Alternation.h>
#include <regexp/Concatenation.h>
#include <regexp/Iteration.h>
#include <regexp/RegExpSymbol.h>
#include <regexp/RegExpEmpty.h>
#include <regexp/RegExpEpsilon.h>

#include <AlibException.h>

namespace conversions
{

class GlushkovTraversal
{
public:
    struct Neighbours
    {
        const regexp::RegExpSymbol * m_first, * m_second;

        Neighbours( const regexp::RegExpSymbol * first, const regexp::RegExpSymbol * second );
        bool operator<( const Neighbours & x ) const;
    };

    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::RegExp & re );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::RegExp & re );
    static std::set<Neighbours> getNeighbours( const regexp::RegExp & re );

private:
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::RegExpElement * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::Alternation * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::Concatenation * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::Iteration * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::RegExpSymbol * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::RegExpEmpty * node );
    static std::set<const regexp::RegExpSymbol*> getLeftmostSymbolsInTree( const regexp::RegExpEpsilon * node );

    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::RegExpElement * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::Alternation * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::Concatenation * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::Iteration * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::RegExpSymbol * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::RegExpEmpty * node );
    static std::set<const regexp::RegExpSymbol*> getRightmostSymbolsInTree( const regexp::RegExpEpsilon * node );

    static std::set<Neighbours> getNeighbours( const regexp::RegExpElement * node );
    static std::set<Neighbours> getNeighbours( const regexp::Alternation * node );
    static std::set<Neighbours> getNeighbours( const regexp::Concatenation * node );
    static std::set<Neighbours> getNeighbours( const regexp::Iteration * node );
    static std::set<Neighbours> getNeighbours( const regexp::RegExpSymbol * node );
    static std::set<Neighbours> getNeighbours( const regexp::RegExpEmpty * node );
    static std::set<Neighbours> getNeighbours( const regexp::RegExpEpsilon * node );
};

} /* namespace conversions */

#endif /* GLUSHKOVTRAVERSAL_H_ */
