#include "AutomatonParser.h"
#include "std/istream.h"
#include <string>
#include <sstream>
#include <vector>
#include <algorithm>

#include "alphabet/Symbol.h"

AutomatonParser::AutomatonParser(std::istream& in) : m_In(in) {

}

automaton::FSM AutomatonParser::parse() {
  automaton::FSM res = automaton::FSM();
  std::string headerLine;
  std::getline(m_In, headerLine);
  
  std::stringstream header(headerLine);
  if(!(header >> "NFA") && !(header.clear(), header >> "DFA")) {
    throw ParseError();
  }
  
  std::string alpha;
  std::vector<alphabet::Symbol> symbols;
  while(header >> alpha) {
    if(alpha != "\\eps") {
      res.addInputSymbol(alphabet::Symbol(alpha));
      symbols.push_back(alphabet::Symbol(alpha));
    } else {
      symbols.push_back(alphabet::Symbol(""));
    }
  }
  
  std::string line;
  while(std::getline(m_In, line)) {
    bool initial = false;
    bool final = false;
    
    std::stringstream linestream(line);
    
    if(linestream >> ">") initial = true;
    else linestream.clear();
    if(linestream >> "<") final = true;
    else linestream.clear();
    
    std::string from;
    linestream >> from;
    
    if(res.getStates().find(from) == res.getStates().end()) res.addState(from);
    if(initial) res.addInitialState(from);
    if(final) res.addFinalState(from);
    
    std::vector<alphabet::Symbol>::const_iterator iter = symbols.begin();
    std::string to;
    while(linestream >> to) {
      if(iter == symbols.end()) throw ParseError();
      if(to != "-") {
      
	std::replace(to.begin(), to.end(), '|', ' ');
	std::stringstream states(to);
	std::string state;
	while(states >> state) {
	  if(res.getStates().find(state) == res.getStates().end()) res.addState(state);
	  
	  res.addTransition(from, *iter, state);
	}
	
      }
      iter++;
    }
  }
  
  return  res;
}