#include "RHDPDADeterminizationUtils.h"

namespace determinization {

const Symbol RHDPDADeterminizationUtils::BOTTOM_OF_STACK_SYMBOL("_"); // TODO specialni symbol pro dno zasobniku zpusobuje chybu


string RHDPDADeterminizationUtils::buildPairName(const StateSymbolPair& pair) {
    return "('" + pair.state.getName() + "', '" + pair.symbol.getSymbol() + "')";;
}


string RHDPDADeterminizationUtils::buildStateName(const SComponent& s, const RComponent& r) {
    string name = "{";
    for (set<PairOfPairs>::const_iterator pairOfPairs = s.pairsOfPairs.begin();
            pairOfPairs != s.pairsOfPairs.end();
            pairOfPairs++) {
        if (pairOfPairs != s.pairsOfPairs.begin()) {
            name += ", ";
        }
        name += "(" + buildPairName(pairOfPairs->pair1) + ", " + buildPairName(pairOfPairs->pair2) + ")";
    }
    name += "}, {";
    for (set<StateSymbolPair>::const_iterator pair = r.pairs.begin();
            pair != r.pairs.end();
            pair++) {
        if (pair != r.pairs.begin()) {
            name += ", ";
        }
        name += buildPairName(*pair);
    }
    return name + "}";
}


const State& RHDPDADeterminizationUtils::getOrCreateState(const SComponent& s, const RComponent& r,
        map<string, StateData>& states, PDA& rdpda) {
    string stateName = buildStateName(s, r);

    map<string, StateData>::iterator statesIter = states.find(stateName);
    if (statesIter != states.end()) {
        return statesIter->second.state;
    }
    
    State state(stateName);
    StateData stateData(state, s, r);
    StateData& insertedData = states.insert(pair<string, StateData>(stateName, stateData)).first->second;
    rdpda.addState(state);
    return insertedData.state;
}


const SComponent RHDPDADeterminizationUtils::getSComponentWithPairsIdentity(const set<StateSymbolPair>& pairs) {
    set<PairOfPairs> pairsOfPairs;
    for (set<StateSymbolPair>::const_iterator pair = pairs.begin(); pair != pairs.end(); pair++) {
        pairsOfPairs.insert(PairOfPairs(*pair, *pair));
    }
    return SComponent(pairsOfPairs);
}


string RHDPDADeterminizationUtils::buildStackSymbolName(const SComponent& s, const RComponent& r,
        const Symbol& input, PDA& rdpda) {
    string name = "(" + buildStateName(s, r) + ")";
    name += ", " + input.getSymbol();
    try {
        rdpda.addStackSymbol(name);
    } catch (AutomatonException e) { /* Stack symbol already exists */ }
    return name;
}


vector<SComponent> RHDPDADeterminizationUtils::generateAllPossibleSComponents(set<StateSymbolPair>& pairs) {
     // Generate all possible pairs of pairs
    vector<PairOfPairs> possiblePairsOfPairs;
    for (set<StateSymbolPair>::iterator pair1 = pairs.begin(); pair1 != pairs.end(); pair1++) {
        for (set<StateSymbolPair>::iterator pair2 = pairs.begin(); pair2 != pairs.end(); pair2++) {
            possiblePairsOfPairs.push_back(PairOfPairs(*pair1, *pair2));
        }
    }
    
    // Generate all possible s components
    vector<SComponent> possibleS;
    int possibleSSize = pow(2, possiblePairsOfPairs.size());
    for (int i = 0; i < possibleSSize; i++) {
        set<PairOfPairs> pairsOfPairsSet;
        int mask = 1;
        for (vector<PairOfPairs>::iterator pairOfPairs = possiblePairsOfPairs.begin();
                pairOfPairs != possiblePairsOfPairs.end();
                pairOfPairs++) {
            if ((i & mask) != 0) {
                pairsOfPairsSet.insert(*pairOfPairs);
            }
            mask <<= 1;
        }
        possibleS.push_back(SComponent(pairsOfPairsSet));
    }

    return possibleS;
}


vector<RComponent> RHDPDADeterminizationUtils::generateAllPossibleRComponents(set<StateSymbolPair>& pairs) {
    // Generate all possible r components
    vector<RComponent> possibleR;
    int possibleRSize = pow(2, pairs.size());
    for (int i = 0; i < possibleRSize; i++) {
        set<StateSymbolPair> pairsSet;
        int mask = 1;
        for (set<StateSymbolPair>::iterator pair = pairs.begin(); pair != pairs.end(); pair++) {
            if ((i & mask) != 0) {
                pairsSet.insert(*pair);
            }
            mask <<= 1;
        }
        possibleR.push_back(RComponent(pairsSet));
    }

    return possibleR;
}


StateData* RHDPDADeterminizationUtils::getUnmarkedState(map<string, StateData>& states) {
    StateData* unmarkedStateData = NULL;
    for (map<string, StateData>::iterator stateIter = states.begin(); stateIter != states.end(); stateIter++) {
        if (!stateIter->second.isMarked) {
            unmarkedStateData = &stateIter->second;
            unmarkedStateData->isMarked = true;
            return unmarkedStateData;
        }
    }
    return NULL;
}


set<StateSymbolPair> RHDPDADeterminizationUtils::combineStatesWithSymbol(const set<State>& states, const Symbol& symbol) {
    set<StateSymbolPair> pairs;
    for (set<State>::iterator state = states.begin(); state != states.end(); state++) {
        pairs.insert(StateSymbolPair(*state, symbol));
    }
    return pairs;
}


set<StateSymbolPair> RHDPDADeterminizationUtils::buildAllStateSymbolPairs(PDA& rhdpda) {
    set<StateSymbolPair> allStateSymbolPairs;
    const set<State>& states = rhdpda.getStates();
    const set<Symbol>& stackAlphabet = rhdpda.getStackAlphabet();
    for (set<State>::iterator state = states.begin(); state != states.end(); state++) {
        for (set<Symbol>::iterator symbol = stackAlphabet.begin(); symbol != stackAlphabet.end(); symbol++) {
            allStateSymbolPairs.insert(StateSymbolPair(*state, *symbol));
        }
    }
    return allStateSymbolPairs;
}

} /* namespace determinization */
