#ifndef VPADETERKMINIZATIONSTRUCTS_H_
#define VPADETERKMINIZATIONSTRUCTS_H_

#include <set>

using namespace automaton;

namespace determinization {

struct StatesPair {
    State state1;
    State state2;
    
    StatesPair(const State& state1, const State& state2)
            : state1(state1),
              state2(state2) {}
    
    bool operator < (const StatesPair& other) const {
        return (state1 < other.state1) || ((state1 == other.state1) && (state2 < other.state2));
    }
    
    bool operator == (const StatesPair& other) const {
        return (state1 == other.state1) && (state2 == other.state2);
    }
    
    bool operator != (const StatesPair& other) const {
        return (state1 != other.state1) || (state2 != other.state2);
    }
};


struct SComponent {
    set<StatesPair> statesPairs;
    SComponent() {}
    SComponent(const set<StatesPair>& statesPairs)
            : statesPairs(statesPairs) {}
};


struct RComponent {
    set<State> states;
    RComponent() {}
    RComponent(const set<State>& states)
            : states(states) {}
};


struct StateData {
    State state;
    SComponent s;
    RComponent r;
    bool isMarked;
    
    StateData(const State& state, const SComponent& s, const RComponent& r)
            : state(state),
              s(s),
              r(r),
              isMarked(false) {}
    
    StateData(const State& state, const SComponent& s)
            : state(state),
              s(s),
              isMarked(false) {}
};


struct PartialStackData {
    SComponent s;
    RComponent r;
    
    PartialStackData(const SComponent& s, const RComponent& r)
            : s(s),
              r(r) {}
};

} /* namespace determinization */
#endif /* VPADETERKMINIZATIONSTRUCTS_H_ */