/*
 * Automaton.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: martin
 */

#include "Automaton.h"

#include <set>
#include "exception/AutomatonException.h"

namespace automaton {

Automaton::~Automaton() {

}

void Automaton::addState(const State& state) {
	pair<set<State>::iterator, bool> ret = states.insert(state);
	if (!ret.second)
		throw AutomatonException("State \"" + state.getName() + "\" already exists.");
}

const set<State>& Automaton::getStates() const {
	return states;
}

void Automaton::addInputSymbol(const Symbol& symbol) {
	pair<set<Symbol>::iterator, bool> ret = inputAlphabet.insert(symbol);
	if (!ret.second)
		throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" already exists.");
}

const set<Symbol>& Automaton::getInputAlphabet() const {
	return inputAlphabet;
}

void Automaton::addInitialState(const State& state) {
	pair<set<State>::iterator, bool> ret = initialStates.insert(state);
	if (!ret.second)
		throw AutomatonException("State \"" + state.getName() + "\" is already initial state.");
}

void Automaton::removeInitialState(const State& state) {
	if (states.find(state) == states.end()) {
		throw AutomatonException("State cannot be set as initial state. It is not present in the automaton.");
	}

	int removed = initialStates.erase(state);
	if (!removed)
		throw AutomatonException("State \"" + state.getName() + "\" isn't initial state.");

}

const set<State>& Automaton::getInitialStates() const {
	return initialStates;
}

void Automaton::addFinalState(const State& state) {
	if (states.find(state) == states.end()) {
		throw AutomatonException("State cannot be set as final state. It is not present in the automaton.");
	}

	pair<set<State>::iterator, bool> ret = finalStates.insert(state);
	if (!ret.second)
		throw AutomatonException("State \"" + state.getName() + "\" is already final state.");
}

void Automaton::removeFinalState(const State& state) {
	int removed = finalStates.erase(state);
	if (!removed)
		throw AutomatonException("State \"" + state.getName() + "\" isn't final state.");
}

const set<State>& Automaton::getFinalStates() const {
	return finalStates;
}

const State& Automaton::createUniqueState(const string& name, bool integerSuffix){
	try{
		State uniqueState(name);
		addState(uniqueState);
		return * states.find(uniqueState);
	}
	catch(AutomatonException& e){
	}

	int i = 0;
	string str = name;

	while(i < INT_MAX)
	{
		if(integerSuffix)
			str = name + std::to_string(i++);
		else
			str += '\'';

		State uniqueState(str);
		try{
			addState(uniqueState);
		}
		catch(AutomatonException& e){
			continue;
		}

		return * states.find(uniqueState);
	}

	throw AutomatonException("Could not create unique state with name " + name + "." );
}

ostream& operator <<(ostream& out, const Automaton& automaton) {
	automaton.toXML(out);
	return out;
}

} /* namespace automaton */


