/*
 * TransitionPDA.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: martin
 */

#include "TransitionPDA.h"

namespace automaton {

TransitionPDA::TransitionPDA(const State& from, const Symbol& input, const State& to) : Transition(from, input, to){

}

TransitionPDA::TransitionPDA(const State& from, const Symbol& input, const State& to, const list<Symbol>& pop,
		const list<Symbol>& push) :
		Transition(from, input, to), pop(pop), push(push) {

}

void TransitionPDA::setPop(const list<Symbol>& pop) {
	this->pop = pop;
}

const list<Symbol>& TransitionPDA::getPop() const {
	return pop;
}

void TransitionPDA::setPush(const list<Symbol>& push) {
	this->push = push;
}

const list<Symbol>& TransitionPDA::getPush() const {
	return push;
}

bool TransitionPDA::operator <(const TransitionPDA& other) const {
	if (from != other.from) {
		return from < other.from;
	} else if (input != other.input) {
		return input < other.input;
	} else if (to != other.to) {
		return to < other.to;
	}

	list<Symbol>::const_iterator it;
	list<Symbol>::const_iterator it2;

	//compare pop size
	if (pop.size() != other.pop.size())
		return pop.size() < other.pop.size();

	//compare pop content
	it = pop.begin();
	it2 = other.pop.begin();
	while (it != pop.end()) {
		if ((*it) != (*it2))
			return (*it) < (*it2);
		it++;
		it2++;
	}

	//compare push size
	if (push.size() != other.push.size())
		return push.size() < other.push.size();

	//compare push content
	it = push.begin();
	it2 = other.push.begin();
	while (it != push.end()) {
		if ((*it) != (*it2))
			return (*it) < (*it2);
		it++;
		it2++;
	}

	return false;
}

bool TransitionPDA::operator ==(const TransitionPDA& other) const {
	if (from == other.from && input == other.input && to == other.to) {

		//compare pop size
		if (pop.size() != other.pop.size())
			return false;
		//compare push size
		if (push.size() != other.push.size())
			return false;

		list<Symbol>::const_iterator it;
		list<Symbol>::const_iterator it2;

		//compare pop content
		it = pop.begin();
		it2 = other.pop.begin();
		while (it != pop.end()) {
			if ((*it) != (*it2))
				return false;
			it++;
			it2++;
		}

		//compare push content
		it = push.begin();
		it2 = other.push.begin();
		while (it != push.end()) {
			if ((*it) != (*it2))
				return false;
			it++;
			it2++;
		}
		return true;
	} else {
		return false;
	}
}

bool TransitionPDA::operator !=(const TransitionPDA& other) const {
	return !((*this) == other);
}

std::ostream& TransitionPDA::operator>>(std::ostream& out) const {
	bool first;
	out << "TransitionPDA from = " << this->from
		<< " to = " << this->to
		<< " input = " << this->input
		<< " pop = [";

	first = true;
	for(list<Symbol>::const_iterator iter = this->pop.begin(); iter != this->pop.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}

	out << "] push = [";

	first = true;
	for(list<Symbol>::const_iterator iter = this->push.begin(); iter != this->push.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}
	out << "]";

	return out;
}

} /* namespace automaton */
