/*
 * ContextSensitiveGrammar.h
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#ifndef CONTEXTSENSITIVEGRAMMAR_H_
#define CONTEXTSENSITIVEGRAMMAR_H_

#include "../Grammar.h"

namespace grammar {

/**
 * Context-sensitive grammar. Type 1 in Chomsky hierarchy.
 */
class ContextSensitiveGrammar: public Grammar {
private:
	/**
	 * Checks that list of symbols specified by prefixStart and prefixEnd on the left side
	 * is also present on the start of right side of the Rule.
	 * @param prefixStart iterator pointing to start of the left side prefix
	 * @param prefixEnd iterator pointing after last symbol of the prefix on the left side
	 * @param rightSide iterator pointing to the first symbol on the right side of the rule
	 * @return true when start of the left side of the rule matches start on the right side of the rule
	 */
	bool startsWith(list<Symbol>::const_iterator prefixStart, list<Symbol>::const_iterator prefixEnd, list<Symbol>::const_iterator rightSide ) const;

	/**
	 * Checks that list of symbols specified by suffixStart and suffixEnd on the left side
	 * is also present at the end of right side of the Rule.
	 * @param suffixStart iterator pointing to the start of left side suffix
	 * @param suffixEnd iterator pointing after the last symbol of the left side of the rule
	 * @param rightSide iterator pointing after the last symbol of the rigt side of the rule
	 */
	bool endsWith(list<Symbol>::const_iterator suffixStart, list<Symbol>::const_iterator suffixEnd, list<Symbol>::const_iterator rightSideEnd ) const;
protected:
	/**
	 * @copydoc Grammar::isValidRule(const Rule&) const
	 */
	bool isValidRule(const Rule& rule) const;
public:
	ContextSensitiveGrammar();
	ContextSensitiveGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols, const Symbol& startSymbol);
};

} /* namespace grammar */
#endif /* CONTEXTSENSITIVEGRAMMAR_H_ */
