/*
 * RegExpPrinter.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "RegExpPrinter.h"

namespace regexp {

const string RegExpPrinter::INDENTATION = "\t";

void RegExpPrinter::toXML(RegExp& regexp, ostream& out) {
	out << "<regexp>\n";
	printElement(regexp.getRegExp(), out, INDENTATION);
	out << "</regexp>\n";
}

void RegExpPrinter::printElement(RegExpElement* element, ostream& out, string prefix) {
	Alternation* alternation = dynamic_cast<Alternation*>(element);
	if (alternation) {
		printAlternation(alternation, out, prefix);
		return;
	}

	Concatenation* concatenation = dynamic_cast<Concatenation*>(element);
	if(concatenation) {
		printConcatenation(concatenation,out, prefix);
		return;
	}

	Iteration* iteration = dynamic_cast<Iteration*>(element);
	if (iteration) {
		printIteration(iteration, out, prefix);
		return;
	}

	RegExpEpsilon* epsilon = dynamic_cast<RegExpEpsilon*>(element);
	if (epsilon) {
		printEpsilon(epsilon, out, prefix);
		return;
	}

	RegExpEmpty* empty = dynamic_cast<RegExpEmpty*>(element);
	if (empty) {
		printEmpty(empty, out, prefix);
		return;
	}

	RegExpSymbol* symbol = dynamic_cast<RegExpSymbol*>(element);
	if (symbol) {
		printSymbol(symbol, out, prefix);
		return;
	}

}

void RegExpPrinter::printContent(list<RegExpElement*>& content, ostream& out, string prefix) {
	for (auto element : content) {
		printElement(element, out, prefix);
	}
}

void RegExpPrinter::printAlternation(Alternation* alternation, ostream& out, string prefix) {
	out << prefix << "<alternation>\n";
	printContent(alternation->getElements(), out, prefix + INDENTATION);
	out << prefix << "</alternation>\n";
}

void RegExpPrinter::printConcatenation(Concatenation* concatenation, ostream& out, string prefix) {
	out << prefix <<"<concatenation>\n";
	printContent(concatenation->getElements(), out, prefix + INDENTATION);
	out << prefix <<"</concatenation>\n";

}

void RegExpPrinter::printIteration(Iteration* iteration, ostream& out, string prefix) {
	out << prefix << "<iteration>\n";
	printElement(iteration->getElement(), out, prefix + INDENTATION);
	out << prefix << "</iteration>\n";
}

void RegExpPrinter::printSymbol(RegExpSymbol* symbol, ostream& out, string prefix) {
	out << prefix << "<symbol>";
	out << symbol->getSymbol();
	out << "</symbol>\n";
}

void RegExpPrinter::printEpsilon(RegExpEpsilon* symbol, ostream& out, string prefix) {
	out << prefix << "<epsilon>";
	out << "</epsilon>\n";
}

void RegExpPrinter::printEmpty(RegExpEmpty* symbol, ostream& out, string prefix) {
	out << prefix << "<empty>";
	out << "</empty>\n";
}

} /* namespace regexp */

