/*
 * UnreachableStateRemover.cpp
 *
 *  Created on: 18. 1. 2014
 *      Author: tomas
 */

#include "UnreachableStateRemover.h"

using namespace alib;
using namespace automaton;
using namespace std;
using namespace trim;

namespace trim
{

FSM UnreachableStateRemover::remove( const FSM & fsm )
{
    if( fsm.getInitialStates( ).size( ) != 1 )
        throw AlibException( "NFA must have exactly one initial state." );

    set<State> reachableStates = findReachableStates( fsm );

    FSM ret;

    for( auto const & q : reachableStates )
        ret.addState( q );

    for( auto const & t : fsm.getTransitions( ) )
    {
        if( isInSet( t.getFrom( ), reachableStates ) )
        {
            if( ! isInSet( t.getInput( ), ret.getInputAlphabet( ) ) )
                ret.addInputSymbol( t.getInput( ) );
            ret.addTransition( t );
        }
    }


    for( auto const & state : fsm.getInitialStates( ) )
        ret.addInitialState( state );


    set<State> intersect;
    const set<State> & F = fsm.getFinalStates( );
    set_intersection( F.begin(), F.end(), reachableStates.begin(), reachableStates.end(), inserter( intersect, intersect.begin() ) );

    for( auto const & state : intersect )
        ret.addFinalState( state );

    return ret;
}

set<State> UnreachableStateRemover::findReachableStates( const FSM & fsm )
{
    set<State> qprev, qcurr;
    qcurr.insert( * fsm.getInitialStates().begin() );

    do
    {
        qprev = qcurr;

        for( auto const & p : qprev )
            for( auto const & transition : fsm.getTransitionsFromState( p ) )
                qcurr.insert( transition.getTo() );
    }
    while( qcurr != qprev );

    return qcurr;
}

} /* namespace trim */
