    #include <iostream>

#include <AutomatonFactory.h>
#include <AlibException.h>
#include <automaton/AutomatonParser.h>

#include <sax/SaxInterface.h>
#include <sax/ParserException.h>

#include "TrimNFA.h"

using namespace std;
using namespace automaton;
using namespace alib;
using namespace sax;

using namespace trim;

int main(int argc, char** argv) {
    int fileParameterIndex = -1;

    try {
        if( argc > 1 )
        {
            for( int i = 1; i < argc; i++ )
            {
                if( string( "-h" ).compare( argv[i] ) == 0 )
                {
                    std::cout << "Removes unreachable states from NFA." << std::endl;
                    std::cout << "Usage: atrim [automaton.xml]" << std::endl;
                    return 1;
                }
                else
                {
                    if(fileParameterIndex == -1)
                        fileParameterIndex = i;
                    else
                        throw AlibException("Only one file can be passed as parameter - " + string(argv[i]) + " " + string(argv[fileParameterIndex]));
                }
            }
        }

        std::list<Token> tokens;

        if(fileParameterIndex != -1)
        {
            SaxInterface::parseFile(argv[fileParameterIndex],tokens);
        }
        else
        {
            string input(istreambuf_iterator<char>(cin), (istreambuf_iterator<char>()));
            SaxInterface::parseMemory(input, tokens);
        }

        UnknownAutomaton automaton = AutomatonParser::parse( tokens );
        FSM fsm = AutomatonFactory::buildFSM( automaton );

        TrimNFA::remove( fsm ).toXML( cout );

    } catch (AlibException& e) {
        cout << e.what() << endl;
        return -1;
    }

    cout.flush();
}
