/*
 * RegExpDerivation.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: tomas
 */

#include "RegExpDerivation.h"

using namespace alib;
using namespace regexp;
using namespace std;

RegExpDerivation::RegExpDerivation( const RegExp & re ) : m_re( re )
{

}

RegExp RegExpDerivation::derivation ( const list<RegExpElement*> & dString ) const
{
    const RegExpElement * oldRegExp = m_re.getRegExp( )->clone( ), * derivedRegExp;

    for( const auto & dSymbol : dString )
    {
        if( dynamic_cast<RegExpEpsilon*>( dSymbol ) )
            continue;
        else if( dynamic_cast<RegExpSymbol*>( dSymbol ) )
            derivedRegExp = derivation( oldRegExp, * dynamic_cast<RegExpSymbol*>( dSymbol ) );
        else
            throw AlibException( "RegExpDerivation::derivation - invalid/unknown RegExpElement passed in dString." );

        delete oldRegExp;
        oldRegExp = derivedRegExp;
    }

    RegExp ret ( oldRegExp );
    delete oldRegExp;
    return ret;
}

RegExpElement * RegExpDerivation::derivation( const RegExpElement * node, const RegExpSymbol & dSymbol ) const
{
    const Alternation * alternation = dynamic_cast<const Alternation*>( node );
    const Concatenation * concatenation = dynamic_cast<const Concatenation*>( node );
    const Iteration * iteration = dynamic_cast<const Iteration*>( node );
    const RegExpSymbol * symbol = dynamic_cast<const RegExpSymbol*>( node );
    const RegExpEpsilon * eps = dynamic_cast<const RegExpEpsilon*>( node );
    const RegExpEmpty * empty = dynamic_cast<const RegExpEmpty*>( node );

    if( alternation )
        return derivation( alternation, dSymbol );
    if( concatenation )
        return derivation( concatenation, dSymbol);
    if( iteration )
        return derivation( iteration, dSymbol );
    if( symbol )
        return derivation( symbol, dSymbol );
    if( eps )
        return derivation( eps, dSymbol );
    if( empty )
        return derivation( empty, dSymbol );

    throw AlibException( "RegExpDerivation::derivation() - unknown RegExpElement node" );
}

RegExpElement * RegExpDerivation::derivation( const Alternation * element, const RegExpSymbol & dSymbol ) const
{
    Alternation* ret = new Alternation( );

    for( const auto & child : element->getElements( ) )
        ret->getElements( ).push_back( derivation( child, dSymbol ) );

    return ret;
}

RegExpElement * RegExpDerivation::derivation( const Concatenation * element, const RegExpSymbol & dSymbol ) const
{
    Alternation* alt = new Alternation( );

    // d/da UV = dU/da ( + dV/da if U.containsEmptyString() )
    for( auto elem = element->getElements( ).begin( ); elem != element->getElements( ).end( ); elem ++ )
    {
        Concatenation* concat = new Concatenation( );
        concat->getElements( ).push_back( derivation( * elem, dSymbol ) );

        auto succeedingElem = elem;
        while( ++ succeedingElem != element->getElements( ).end( ) )
            concat->getElements( ).push_back( ( * succeedingElem )->clone( ) );

        alt->getElements( ).push_back( concat );

        if( ( * elem )->containsEmptyString( ) )
            continue;
        break; // intentionally ( to match algorithm )
    }

    return alt;
}

RegExpElement * RegExpDerivation::derivation( const Iteration * element, const RegExpSymbol & dSymbol ) const
{
    Concatenation* ret = new Concatenation( );

    ret->getElements( ).push_back( derivation( element->getElement( ), dSymbol ) );
    ret->getElements( ).push_back( element->clone( ) );

    return ret;
}

RegExpElement * RegExpDerivation::derivation( const RegExpSymbol * element, const RegExpSymbol & dSymbol ) const
{
    if( dSymbol == element->getSymbol( ) )
        return new RegExpEpsilon( );
    else
        return new RegExpEmpty( );
}

RegExpElement * RegExpDerivation::derivation( const RegExpEpsilon * element, const RegExpSymbol & dSymbol ) const
{
    return new RegExpEmpty( );
}

RegExpElement * RegExpDerivation::derivation( const RegExpEmpty * element, const RegExpSymbol & dSymbol ) const
{
    return new RegExpEmpty( );
}
