/*
 * RegExpDerivation.h
 *
 *  Created on: 19. 1. 2014
 *      Author: tomas
 */

#ifndef REGEXPDERIVATION_H_
#define REGEXPDERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/RegExpElement.h>
#include <regexp/Alternation.h>
#include <regexp/Concatenation.h>
#include <regexp/Iteration.h>
#include <regexp/RegExpSymbol.h>
#include <regexp/RegExpEmpty.h>
#include <regexp/RegExpEpsilon.h>

#include <AlibException.h>

#include <list>


/**
 * Calculates derivation of regular expression.
 *
 * Sources:
 *  - Melichar, definition 2.91 in chapter 2.4.3
 *  - Brzozowski, J. A. - Derivatives of regular expressions (1964)
 */
class RegExpDerivation
{
public:
    RegExpDerivation( const regexp::RegExp & re );
    regexp::RegExp derivation( const std::list<regexp::RegExpElement*> & dString ) const;

private:
    regexp::RegExpElement * derivation( const regexp::RegExpElement * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Alternation * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Concatenation * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Iteration * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpSymbol * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpEpsilon * element, const regexp::RegExpSymbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpEmpty * element, const regexp::RegExpSymbol & dSymbol ) const;

    const regexp::RegExp & m_re;
};

#endif /* REGEXPDERIVATION_H_ */
