#include "RegExpAlphabet.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;
using namespace std;

set<RegExpSymbol> RegExpAlphabet::getSymbols( const RegExp & re )
{
    set<RegExpSymbol> alphabet;

    for( const auto & symbol : getSymbolsListInOrder( re ) )
        alphabet.insert( RegExpSymbol( symbol->getSymbol( ) ) );

    return alphabet;
}

list<const RegExpSymbol*> RegExpAlphabet::getSymbolsListInOrder( const RegExp & re )
{
    list<const RegExpSymbol*> alphabet;
    searchSymbols( re.getRegExp( ), alphabet );
    return alphabet;
}


void RegExpAlphabet::searchSymbols( const RegExpElement * node, list<const RegExpSymbol*> & alphabet )
{
    const Alternation* alternation = dynamic_cast<const Alternation*>( node );
    const Concatenation* concatenation = dynamic_cast<const Concatenation*>( node );
    const Iteration* iteration = dynamic_cast<const Iteration*>( node );
    const RegExpSymbol* symbol = dynamic_cast<const RegExpSymbol*>( node );
    const RegExpEmpty * empty = dynamic_cast<const RegExpEmpty*>( node );
    const RegExpEpsilon* eps = dynamic_cast<const RegExpEpsilon*>( node );

    if( symbol )
    {
        alphabet.push_back( symbol );
        return;
    }

    else if( alternation )
    {
        for( const auto & element : alternation->getElements( ) )
            searchSymbols( element, alphabet );
        return;
    }

    else if( concatenation )
    {
        for( const auto & element : concatenation->getElements( ) )
            searchSymbols( element, alphabet );
        return;
    }

    else if( iteration )
    {
        searchSymbols( iteration->getElement( ), alphabet );
        return;
    }

    else if( empty )
        return;

    else if( eps )
        return;


    throw AlibException( "RegExpAlphabet::traverseSymbols() - unknown RegExpElement node" );
}
